/*
 * Podnikovka.java
 *
 * Created on 10. jen 2004, 14:51
 */

package cz.vse.xkucf03.pocitani.ekonomie;

/**
 * Poty z oboru podnikov ekonomika.
 *
 * Tato tda tvo vpoetn jdro, obsahuje statick metody.
 * Grafick uivatelsk rozhran k tmto metodm nabz tda PodnikovkaGUI
 *
 * @author  x
 */
public class Podnikovka {
    
    
    /**
     * Pokud se zisk rovn nule, pak:
     *      Jedn se o takov objem vroby, pi kterm se trby rovnaj celkovm nkladm,
     *      je to tedy objem vroby pi kterm podnik nen ztrtov, ale tak nedosahuje zisku
     *
     * Pokud je zadn nenulov zisk,
     *      vrac funkce mnostv produkce, pi kter podnik tohoto zisku dosahuje
     */
    public static double BodZvratu(double fixniNaklady, double variabilniNaklady, double cena, double zisk) {
        return (fixniNaklady + zisk) / (cena - variabilniNaklady);
    }
    
    /**
     * Pot variabiln a fixn nklady metodou dvou obdob - ze zadanch obdob vyjdench kombinacemi
     * mnostv a celkovch nklad vybere 2 obdob (s nejvtm a nejmenm mnostvm) a z nich pomoc
     * soustavy dvou rovnic spot poadovan veliiny
     */
    public static FixniVariabilniNaklady FnVnDveObdobi(MnozstviCelkoveNaklady[] MnozstviCelkoveNaklady) {
        // nejdve vybereme dv obdob s nejvt a nejmen produkc
        double maxQ = MnozstviCelkoveNaklady[0].getMnozstvi();
        double maxTC = MnozstviCelkoveNaklady[0].getNaklady();
        double minQ = MnozstviCelkoveNaklady[0].getMnozstvi();
        double minTC = MnozstviCelkoveNaklady[0].getNaklady();
        
        for(int i = 1; MnozstviCelkoveNaklady.length > i; i ++){
            
            if (maxQ < MnozstviCelkoveNaklady[i].getMnozstvi()) {
                maxQ = MnozstviCelkoveNaklady[i].getMnozstvi();
                maxTC = MnozstviCelkoveNaklady[i].getNaklady();
            }
            
            if (minQ > MnozstviCelkoveNaklady[i].getMnozstvi()) {
                minQ = MnozstviCelkoveNaklady[i].getMnozstvi();
                minTC = MnozstviCelkoveNaklady[i].getNaklady();
            }
            
        }
        
        // spotme fixn a variabiln nklady
        double VN = (maxTC - minTC) / (maxQ - minQ);
        double FN = maxTC - VN*maxQ;
        return new FixniVariabilniNaklady(FN, VN);
        
    }
    
    /**
     * Podobn funkce jako FnVnDveObdobi, ale nevybr obdob s nejvtm a nejmenm mnostvm,
     * ale provd vpoty pro vechny kombinace obdob (bez opakovn)
     * vsledkem je proto pole o dlce n*(n-1)
     */
    public static FixniVariabilniNaklady[] FnVnDveObdobiKomplet(MnozstviCelkoveNaklady[] MnozstviCelkoveNaklady) {
        
        // Vytvome pole, do kterho budeme ukldat vsledky
        FixniVariabilniNaklady vysledek[] = new FixniVariabilniNaklady[MnozstviCelkoveNaklady.length * (MnozstviCelkoveNaklady.length - 1)];
        
        /**
         * i - ta prvnho ze dvou obdob
         * ii - ta druhho ze dvou obdob
         * iii - ta pi ukldn do vsledku
         */
        
        int iii = 0;
        for(int i = 0; MnozstviCelkoveNaklady.length > i; i++){
            MnozstviCelkoveNaklady jednicka = MnozstviCelkoveNaklady[i];
            
            for(int ii = 0; MnozstviCelkoveNaklady.length > ii; ii++){
                if (i != ii) { // vyloume kombinace, kdy porovnvme obdob samo se sebou
                    MnozstviCelkoveNaklady dvojka = MnozstviCelkoveNaklady[ii];
                    vysledek[iii] = FnVnDveObdobi(new MnozstviCelkoveNaklady[]{jednicka, dvojka});
                    iii++;
                }
            }
        }
        return vysledek;
    }
    
    /** zkuebn metoda - smazat!*/
    public static void main(String args[]) {
        MnozstviCelkoveNaklady MnozstviCelkoveNaklady[] = new MnozstviCelkoveNaklady[4];
        MnozstviCelkoveNaklady[0] = new MnozstviCelkoveNaklady(6224,6967);
        MnozstviCelkoveNaklady[1] = new MnozstviCelkoveNaklady(8460,7776);
        MnozstviCelkoveNaklady[2] = new MnozstviCelkoveNaklady(10408,8002);
        MnozstviCelkoveNaklady[3] = new MnozstviCelkoveNaklady(12623,8687);
        
        FixniVariabilniNaklady vysledek = Podnikovka.FnVnDveObdobi(MnozstviCelkoveNaklady);
        System.out.println(vysledek.getFixni());
        System.out.println(vysledek.getVariabilni());
        
        FixniVariabilniNaklady[] vysledek2 = Podnikovka.FnVnDveObdobiKomplet(MnozstviCelkoveNaklady);
        
        for (int i = 0; vysledek2.length > i; i++){
            System.out.println("--- " + i + " --------------------------");
            System.out.println(vysledek2[i].getFixni());
            System.out.println(vysledek2[i].getVariabilni());
            
            
        }
        
        System.out.println("WACC: " + Podnikovka.WACC(0.15, 0.2, 0.4, 0.65));
        System.out.println(Podnikovka.optimalniVelikostDodavky(130, 10000, 5000));
        
        double mira = 0.2f;
        int pocet = 40;
        double SH = 44625.34628654436;
        System.out.println(Podnikovka.urocitelBH(Podnikovka.odurocitelSH(SH, mira, pocet), mira, pocet));
        System.out.println(Podnikovka.rocniUrokovaMira(0.03, 4));
        System.out.println(Podnikovka.umorovatelAnuita(200000,0.1,5));
        System.out.println(Podnikovka.umorovatelAnuita(200000,Podnikovka.neRocniUrokovaMira(0.1,12),5*12));
        System.out.println(Math.pow(10000, (1/12)));
        System.out.println(Podnikovka.neRocniUrokovaMira(0.6,12));
        
        
    }
    
    /** Nklady na vlastn kapitl = Pvk = (dividenda / cenaAkcie) + miraRustuDividend */
    public static double nakladyNaVK(double dividenda, double cenaAkcie, double miraRustuDividend) {
        return (dividenda / cenaAkcie) + miraRustuDividend;
    }
    
    /**
     *  Ven prmrn kapitlov nklady - nklady na kapitl (uren podnikov diskontn mry).
     *
     *  Nklady na vlastn kapitl = vlastnky poadovan vnos
     *  Nklady na ciz kapitl = rok
     *
     *  WACC se pouv jako diskontn mra pro pepoet budoucch hodnot na hodnoty souasn.
     *  Pomoc WACC lze optimalizovat kapitlovou strukturu podniku - takov mra zadluenosti, pi n je WACC minimln.
     */
    public static double WACC(double nakladyNaCK, double nakladyNaVK, double danovaSazba, double podilVK) {
        return nakladyNaCK * (1 - danovaSazba) * (1 - podilVK) + nakladyNaVK * podilVK;
    }
    
    /**
     *  Optimn velikost dodvky - Harris-Willsonv vzorec
     *
     *  rocniDodavka = ron ve celkov dodvky v naturlnch jednotkch
     *  nakladyNaJednuDodavku = nlkady na jednu jednotku zsob
     *  nakladyNaSkladovaniJednotkyZasobZaRok = nlkady na skladovn jednotky zsob v K za rok
     *
     *  rok lze zobecnit na obdob
     *
     *  vrac: optimln velikost dodvky v naturlnch jednotkch
     */
    public static double optimalniVelikostDodavky(double rocniDodavka, double nakladyNaJednuDodavku, double nakladyNaSkladovaniJednotkyZasobZaRok) {
        return Math.sqrt(2 * rocniDodavka * nakladyNaJednuDodavku / nakladyNaSkladovaniJednotkyZasobZaRok);
    }
    
    /**
     *  roitel vyjaduje budouc hodnotu souasn jednorzov jednotkov platby.
     *  To znamen, e s jeho pomoc  lze vypotat, jakou hodnotu bude mt dnes uloen stka.
     *  Za n obdob pi dan rokov me.
     */
    public static double urocitelBH(double soucasnaHodnota, double urokovaMira, int pocetObdobi) {
        return (soucasnaHodnota * Math.pow((1 + urokovaMira), pocetObdobi));
    }
    
    /**
     *  Odroitel vyjaduje souasnou hodnotu budouc jednorzov jednotkov platby.
     *  S jeho pomoc lze vypotat, jakou hodnotu je nutn  dnes uloit,
     *  aby za n obdob pi rokov me i vzrostla na poadovanou stku
     */
    public static double odurocitelSH(double budouciHodnota, double urokovaMira, double pocetObdobi) {
        return (budouciHodnota / Math.pow((1 + urokovaMira), pocetObdobi));
    }
    
    /**
     *  Stadatel vyjaduje budouc hodnotu n jednotkovch pravidelnch plateb.
     *  S jeho pomoc lze vypotat, jak stka bude uspoena za n obdob,
     *  bude-li pravideln ukldna stejn stka (anuita) pi rokov me i.
     */
    public static double stradatelBH(double anuita, double urokovaMira, int pocetObdobi) {
        return anuita * (Math.pow((1 + urokovaMira), pocetObdobi) - 1) / urokovaMira;
    }
    
    /**
     *  Fondovatel umon stanovit, jakou stku je nutn pravideln pi rokov me i  ukldat,
     *  aby byla dosaena poadovan suma
     */
    public static double fondovatelAnuita(double budouciHodnota, double urokovaMira, int pocetObdobi) {
        return budouciHodnota * urokovaMira / (Math.pow((1 + urokovaMira),pocetObdobi) - 1);
    }
    
    /**
     *  Zsobitel vyjaduje souasnou hodnotu n jednotkovch pravidelnch plateb.
     *  S jeho pomoc lze stanovit, jak stka mus bt dnes uloena,
     *  aby z n byl po n obdob pi rokov me i vyplcen pravideln dchod.
     */
    public static double zasobitelSH(double anuita, double urokovaMira, int pocetObdobi) {
        return anuita * (Math.pow((1 + urokovaMira), pocetObdobi) - 1) / (urokovaMira * Math.pow((1 + urokovaMira), pocetObdobi));
    }
    
    
    
    /**
     *  Pepotv denn, msn nebo tvrtletn rokovou mru na ron
     *  pocetObdobiVRoce = 365 pro denn, 12 pro msn, 4 pro ctvrtletn...
     *
     *  Pklad: Ron rokov mra 12,551 % je ekvivalentn tvrtletn rokov me ve vi 3 %.
     */
    public static double rocniUrokovaMira(double vychoziUrokovaMira, int pocetObdobiVRoce) {
        return Math.pow((1 + vychoziUrokovaMira), pocetObdobiVRoce) - 1;
    }
    
    /**
     *  Pepotv ron rokovou mru na denn, msn nebo tvrtletn
     *  pocetObdobiVRoce = 365 pro denn, 12 pro msn, 4 pro ctvrtletn...
     *
     * inverzn funkce k funkci rocniUrokovaMira
     */
    public static double neRocniUrokovaMira(double rocniUrokovaMira, int pocetObdobiVRoce) {
        //TODO nefunguje odmocnovn
        return Math.pow((1 + rocniUrokovaMira), (1 / pocetObdobiVRoce)) - 1;
    }
    
    /**
     *  Umoovatel umon stanovit velikost pravidelnch, stejn velkch spltek,
     *  pokud pi rokov me i za n obdob je nutn splatit pjenou stku penz.
     */
    public static double umorovatelAnuita(double soucasnaHodnota, double urokovaMira, int pocetObdobi) {
        return soucasnaHodnota * ((urokovaMira * Math.pow((1 + urokovaMira), pocetObdobi)) / (Math.pow((1 + urokovaMira), pocetObdobi) - 1));
    }
    
}

